@@ -32,4 +32,4 @@ class Signal(FrozenList):
             raise RuntimeError("Cannot send non-frozen signal.")
 
         for receiver in self:
-            await receiver(*args, **kwargs)
+            await receiver(*args, **kwargs)  # type: ignore
