@@ -1,30 +1,42 @@
 import asyncio
 import collections
+import warnings
+from typing import List  # noqa
+from typing import Awaitable, Callable, Generic, Optional, Tuple, TypeVar
 
-from .helpers import set_exception, set_result
+from .base_protocol import BaseProtocol
+from .helpers import BaseTimerContext, set_exception, set_result
 from .log import internal_logger
 
 
+try:  # pragma: no cover
+    from typing import Deque  # noqa
+except ImportError:
+    from typing_extensions import Deque  # noqa
+
+
 __all__ = (
     'EMPTY_PAYLOAD', 'EofStream', 'StreamReader', 'DataQueue',
     'FlowControlDataQueue')
 
 DEFAULT_LIMIT = 2 ** 16
 
+_T = TypeVar('_T')
+
 
 class EofStream(Exception):
     """eof stream indication."""
 
 
-class AsyncStreamIterator:
+class AsyncStreamIterator(Generic[_T]):
 
-    def __init__(self, read_func):
+    def __init__(self, read_func: Callable[[], Awaitable[_T]]) -> None:
         self.read_func = read_func
 
-    def __aiter__(self):
+    def __aiter__(self) -> 'AsyncStreamIterator[_T]':
         return self
 
-    async def __anext__(self):
+    async def __anext__(self) -> _T:
         try:
             rv = await self.read_func()
         except EofStream:
@@ -34,9 +46,16 @@ class AsyncStreamIterator:
         return rv
 
 
-class ChunkTupleAsyncStreamIterator(AsyncStreamIterator):
-    async def __anext__(self):
-        rv = await self.read_func()
+class ChunkTupleAsyncStreamIterator:
+
+    def __init__(self, stream: 'StreamReader') -> None:
+        self._stream = stream
+
+    def __aiter__(self) -> 'ChunkTupleAsyncStreamIterator':
+        return self
+
+    async def __anext__(self) -> Tuple[bytes, bool]:
+        rv = await self._stream.readchunk()
         if rv == (b'', False):
             raise StopAsyncIteration  # NOQA
         return rv
@@ -44,32 +63,32 @@ class ChunkTupleAsyncStreamIterator(AsyncStreamIterator):
 
 class AsyncStreamReaderMixin:
 
-    def __aiter__(self):
-        return AsyncStreamIterator(self.readline)
+    def __aiter__(self) -> AsyncStreamIterator[bytes]:
+        return AsyncStreamIterator(self.readline)  # type: ignore
 
-    def iter_chunked(self, n):
+    def iter_chunked(self, n: int) -> AsyncStreamIterator[bytes]:
         """Returns an asynchronous iterator that yields chunks of size n.
 
         Python-3.5 available for Python 3.5+ only
         """
-        return AsyncStreamIterator(lambda: self.read(n))
+        return AsyncStreamIterator(lambda: self.read(n))  # type: ignore
 
-    def iter_any(self):
+    def iter_any(self) -> AsyncStreamIterator[bytes]:
         """Returns an asynchronous iterator that yields all the available
         data as soon as it is received
 
         Python-3.5 available for Python 3.5+ only
         """
-        return AsyncStreamIterator(self.readany)
+        return AsyncStreamIterator(self.readany)  # type: ignore
 
-    def iter_chunks(self):
+    def iter_chunks(self) -> ChunkTupleAsyncStreamIterator:
         """Returns an asynchronous iterator that yields chunks of data
         as they are received by the server. The yielded objects are tuples
         of (bytes, bool) as returned by the StreamReader.readchunk method.
 
         Python-3.5 available for Python 3.5+ only
         """
-        return ChunkTupleAsyncStreamIterator(self.readchunk)
+        return ChunkTupleAsyncStreamIterator(self)  # type: ignore
 
 
 class StreamReader(AsyncStreamReaderMixin):
@@ -88,8 +107,10 @@ class StreamReader(AsyncStreamReaderMixin):
 
     total_bytes = 0
 
-    def __init__(self, protocol,
-                 *, limit=DEFAULT_LIMIT, timer=None, loop=None):
+    def __init__(self, protocol: BaseProtocol,
+                 *, limit: int=DEFAULT_LIMIT,
+                 timer: Optional[BaseTimerContext]=None,
+                 loop: Optional[asyncio.AbstractEventLoop]=None) -> None:
         self._protocol = protocol
         self._low_water = limit
         self._high_water = limit * 2
@@ -98,17 +119,17 @@ class StreamReader(AsyncStreamReaderMixin):
         self._loop = loop
         self._size = 0
         self._cursor = 0
-        self._http_chunk_splits = None
-        self._buffer = collections.deque()
+        self._http_chunk_splits = None  # type: Optional[List[int]]
+        self._buffer = collections.deque()  # type: Deque[bytes]
         self._buffer_offset = 0
         self._eof = False
-        self._waiter = None
-        self._eof_waiter = None
-        self._exception = None
+        self._waiter = None  # type: Optional[asyncio.Future[bool]]
+        self._eof_waiter = None  # type: Optional[asyncio.Future[bool]]
+        self._exception = None  # type: Optional[BaseException]
         self._timer = timer
-        self._eof_callbacks = []
+        self._eof_callbacks = []  # type: List[Callable[[], None]]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         info = [self.__class__.__name__]
         if self._size:
             info.append('%d bytes' % self._size)
@@ -122,10 +143,10 @@ class StreamReader(AsyncStreamReaderMixin):
             info.append('e=%r' % self._exception)
         return '<%s>' % ' '.join(info)
 
-    def exception(self):
+    def exception(self) -> Optional[BaseException]:
         return self._exception
 
-    def set_exception(self, exc):
+    def set_exception(self, exc: BaseException) -> None:
         self._exception = exc
         self._eof_callbacks.clear()
 
@@ -139,7 +160,7 @@ class StreamReader(AsyncStreamReaderMixin):
             set_exception(waiter, exc)
             self._eof_waiter = None
 
-    def on_eof(self, callback):
+    def on_eof(self, callback: Callable[[], None]) -> None:
         if self._eof:
             try:
                 callback()
@@ -148,7 +169,7 @@ class StreamReader(AsyncStreamReaderMixin):
         else:
             self._eof_callbacks.append(callback)
 
-    def feed_eof(self):
+    def feed_eof(self) -> None:
         self._eof = True
 
         waiter = self._waiter
@@ -169,15 +190,15 @@ class StreamReader(AsyncStreamReaderMixin):
 
         self._eof_callbacks.clear()
 
-    def is_eof(self):
+    def is_eof(self) -> bool:
         """Return True if  'feed_eof' was called."""
         return self._eof
 
-    def at_eof(self):
+    def at_eof(self) -> bool:
         """Return True if the buffer is empty and 'feed_eof' was called."""
         return self._eof and not self._buffer
 
-    async def wait_eof(self):
+    async def wait_eof(self) -> None:
         if self._eof:
             return
 
@@ -188,9 +209,13 @@ class StreamReader(AsyncStreamReaderMixin):
         finally:
             self._eof_waiter = None
 
-    def unread_data(self, data):
+    def unread_data(self, data: bytes) -> None:
         """ rollback reading some data from stream, inserting it to buffer head.
         """
+        warnings.warn("unread_data() is deprecated "
+                      "and will be removed in future releases (#3260)",
+                      DeprecationWarning,
+                      stacklevel=2)
         if not data:
             return
 
@@ -203,7 +228,7 @@ class StreamReader(AsyncStreamReaderMixin):
         self._eof_counter = 0
 
     # TODO: size is ignored, remove the param later
-    def feed_data(self, data, size=0):
+    def feed_data(self, data: bytes, size: int=0) -> None:
         assert not self._eof, 'feed_data after feed_eof'
 
         if not data:
@@ -222,11 +247,11 @@ class StreamReader(AsyncStreamReaderMixin):
                 not self._protocol._reading_paused):
             self._protocol.pause_reading()
 
-    def begin_http_chunk_receiving(self):
+    def begin_http_chunk_receiving(self) -> None:
         if self._http_chunk_splits is None:
             self._http_chunk_splits = []
 
-    def end_http_chunk_receiving(self):
+    def end_http_chunk_receiving(self) -> None:
         if self._http_chunk_splits is None:
             raise RuntimeError("Called end_chunk_receiving without calling "
                                "begin_chunk_receiving first")
@@ -234,7 +259,13 @@ class StreamReader(AsyncStreamReaderMixin):
                 self._http_chunk_splits[-1] != self.total_bytes:
             self._http_chunk_splits.append(self.total_bytes)
 
-    async def _wait(self, func_name):
+            # wake up readchunk when end of http chunk received
+            waiter = self._waiter
+            if waiter is not None:
+                self._waiter = None
+                set_result(waiter, False)
+
+    async def _wait(self, func_name: str) -> None:
         # StreamReader uses a future to link the protocol feed_data() method
         # to a read coroutine. Running two read coroutines at the same time
         # would have an unexpected behaviour. It would not possible to know
@@ -253,7 +284,7 @@ class StreamReader(AsyncStreamReaderMixin):
         finally:
             self._waiter = None
 
-    async def readline(self):
+    async def readline(self) -> bytes:
         if self._exception is not None:
             raise self._exception
 
@@ -283,7 +314,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return b''.join(line)
 
-    async def read(self, n=-1):
+    async def read(self, n: int=-1) -> bytes:
         if self._exception is not None:
             raise self._exception
 
@@ -320,7 +351,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return self._read_nowait(n)
 
-    async def readany(self):
+    async def readany(self) -> bytes:
         if self._exception is not None:
             raise self._exception
 
@@ -329,7 +360,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return self._read_nowait(-1)
 
-    async def readchunk(self):
+    async def readchunk(self) -> Tuple[bytes, bool]:
         """Returns a tuple of (data, end_of_http_chunk). When chunked transfer
         encoding is used, end_of_http_chunk is a boolean indicating if the end
         of the data corresponds to the end of a HTTP chunk , otherwise it is
@@ -346,24 +377,26 @@ class StreamReader(AsyncStreamReaderMixin):
                 return (b"", True)
             await self._wait('readchunk')
 
-        if not self._buffer:
+        if not self._buffer and not self._http_chunk_splits:
             # end of file
             return (b"", False)
         elif self._http_chunk_splits is not None:
             while self._http_chunk_splits:
                 pos = self._http_chunk_splits[0]
                 self._http_chunk_splits = self._http_chunk_splits[1:]
+                if pos == self._cursor:
+                    return (b"", True)
                 if pos > self._cursor:
                     return (self._read_nowait(pos-self._cursor), True)
             return (self._read_nowait(-1), False)
         else:
             return (self._read_nowait_chunk(-1), False)
 
-    async def readexactly(self, n):
+    async def readexactly(self, n: int) -> bytes:
         if self._exception is not None:
             raise self._exception
 
-        blocks = []
+        blocks = []  # type: List[bytes]
         while n > 0:
             block = await self.read(n)
             if not block:
@@ -375,7 +408,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return b''.join(blocks)
 
-    def read_nowait(self, n=-1):
+    def read_nowait(self, n: int=-1) -> bytes:
         # default was changed to be consistent with .read(-1)
         #
         # I believe the most users don't know about the method and
@@ -389,7 +422,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return self._read_nowait(n)
 
-    def _read_nowait_chunk(self, n):
+    def _read_nowait_chunk(self, n: int) -> bytes:
         first_buffer = self._buffer[0]
         offset = self._buffer_offset
         if n != -1 and len(first_buffer) - offset > n:
@@ -411,7 +444,7 @@ class StreamReader(AsyncStreamReaderMixin):
             self._protocol.resume_reading()
         return data
 
-    def _read_nowait(self, n):
+    def _read_nowait(self, n: int) -> bytes:
         chunks = []
 
         while self._buffer:
@@ -427,79 +460,79 @@ class StreamReader(AsyncStreamReaderMixin):
 
 class EmptyStreamReader(AsyncStreamReaderMixin):
 
-    def exception(self):
+    def exception(self) -> Optional[BaseException]:
         return None
 
-    def set_exception(self, exc):
+    def set_exception(self, exc: BaseException) -> None:
         pass
 
-    def on_eof(self, callback):
+    def on_eof(self, callback: Callable[[], None]) -> None:
         try:
             callback()
         except Exception:
             internal_logger.exception('Exception in eof callback')
 
-    def feed_eof(self):
+    def feed_eof(self) -> None:
         pass
 
-    def is_eof(self):
+    def is_eof(self) -> bool:
         return True
 
-    def at_eof(self):
+    def at_eof(self) -> bool:
         return True
 
-    async def wait_eof(self):
+    async def wait_eof(self) -> None:
         return
 
-    def feed_data(self, data):
+    def feed_data(self, data: bytes, n: int=0) -> None:
         pass
 
-    async def readline(self):
+    async def readline(self) -> bytes:
         return b''
 
-    async def read(self, n=-1):
+    async def read(self, n: int=-1) -> bytes:
         return b''
 
-    async def readany(self):
+    async def readany(self) -> bytes:
         return b''
 
-    async def readchunk(self):
-        return (b'', False)
+    async def readchunk(self) -> Tuple[bytes, bool]:
+        return (b'', True)
 
-    async def readexactly(self, n):
+    async def readexactly(self, n: int) -> bytes:
         raise asyncio.streams.IncompleteReadError(b'', n)
 
-    def read_nowait(self):
+    def read_nowait(self) -> bytes:
         return b''
 
 
 EMPTY_PAYLOAD = EmptyStreamReader()
 
 
-class DataQueue:
+class DataQueue(Generic[_T]):
     """DataQueue is a general-purpose blocking queue with one reader."""
 
-    def __init__(self, *, loop=None):
+    def __init__(self, loop: asyncio.AbstractEventLoop) -> None:
         self._loop = loop
         self._eof = False
-        self._waiter = None
-        self._exception = None
+        self._waiter = None  # type: Optional[asyncio.Future[bool]]
+        self._exception = None  # type: Optional[BaseException]
         self._size = 0
-        self._buffer = collections.deque()
+        self._buffer = collections.deque()  # type: Deque[Tuple[_T, int]]
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._buffer)
 
-    def is_eof(self):
+    def is_eof(self) -> bool:
         return self._eof
 
-    def at_eof(self):
+    def at_eof(self) -> bool:
         return self._eof and not self._buffer
 
-    def exception(self):
+    def exception(self) -> Optional[BaseException]:
         return self._exception
 
-    def set_exception(self, exc):
+    def set_exception(self, exc: BaseException) -> None:
         self._eof = True
         self._exception = exc
 
@@ -508,7 +541,7 @@ class DataQueue:
             set_exception(waiter, exc)
             self._waiter = None
 
-    def feed_data(self, data, size=0):
+    def feed_data(self, data: _T, size: int=0) -> None:
         self._size += size
         self._buffer.append((data, size))
 
@@ -517,7 +550,7 @@ class DataQueue:
             self._waiter = None
             set_result(waiter, True)
 
-    def feed_eof(self):
+    def feed_eof(self) -> None:
         self._eof = True
 
         waiter = self._waiter
@@ -525,7 +558,7 @@ class DataQueue:
             self._waiter = None
             set_result(waiter, False)
 
-    async def read(self):
+    async def read(self) -> _T:
         if not self._buffer and not self._eof:
             assert not self._waiter
             self._waiter = self._loop.create_future()
@@ -545,28 +578,30 @@ class DataQueue:
             else:
                 raise EofStream
 
-    def __aiter__(self):
+    def __aiter__(self) -> AsyncStreamIterator[_T]:
         return AsyncStreamIterator(self.read)
 
 
-class FlowControlDataQueue(DataQueue):
+class FlowControlDataQueue(DataQueue[_T]):
     """FlowControlDataQueue resumes and pauses an underlying stream.
 
     It is a destination for parsed data."""
 
-    def __init__(self, protocol, *, limit=DEFAULT_LIMIT, loop=None):
+    def __init__(self, protocol: BaseProtocol, *,
+                 limit: int=DEFAULT_LIMIT,
+                 loop: asyncio.AbstractEventLoop) -> None:
         super().__init__(loop=loop)
 
         self._protocol = protocol
         self._limit = limit * 2
 
-    def feed_data(self, data, size):
+    def feed_data(self, data: _T, size: int=0) -> None:
         super().feed_data(data, size)
 
         if self._size > self._limit and not self._protocol._reading_paused:
             self._protocol.pause_reading()
 
-    async def read(self):
+    async def read(self) -> _T:
         try:
             return await super().read()
         finally:
