@@ -12,7 +12,7 @@ from multidict import CIMultiDict, MultiDict
 from yarl import URL
 
 import aiohttp
-from aiohttp import hdrs, web
+from aiohttp import client, hdrs, web
 from aiohttp.client import ClientSession
 from aiohttp.client_reqrep import ClientRequest
 from aiohttp.connector import BaseConnector, TCPConnector
@@ -21,7 +21,9 @@ from aiohttp.helpers import DEBUG, PY_36
 
 @pytest.fixture
 def connector(loop):
-    conn = BaseConnector(loop=loop)
+    async def make_conn():
+        return BaseConnector(loop=loop)
+    conn = loop.run_until_complete(make_conn())
     proto = mock.Mock()
     conn._conns['a'] = [(proto, 123)]
     yield conn
@@ -32,7 +34,7 @@ def connector(loop):
 def create_session(loop):
     session = None
 
-    def maker(*args, **kwargs):
+    async def maker(*args, **kwargs):
         nonlocal session
         session = ClientSession(*args, loop=loop, **kwargs)
         return session
@@ -42,8 +44,8 @@ def create_session(loop):
 
 
 @pytest.fixture
-def session(create_session):
-    return create_session()
+def session(create_session, loop):
+    return loop.run_until_complete(create_session())
 
 
 @pytest.fixture
@@ -59,60 +61,61 @@ def params():
         read_until_eof=False)
 
 
-def test_close_coro(create_session, loop):
-    session = create_session()
-    loop.run_until_complete(session.close())
+async def test_close_coro(create_session) -> None:
+    session = await create_session()
+    await session.close()
 
 
-def test_init_headers_simple_dict(create_session):
-    session = create_session(headers={"h1": "header1",
-                                      "h2": "header2"})
+async def test_init_headers_simple_dict(create_session) -> None:
+    session = await create_session(headers={"h1": "header1",
+                                            "h2": "header2"})
     assert (sorted(session._default_headers.items()) ==
             ([("h1", "header1"), ("h2", "header2")]))
 
 
-def test_init_headers_list_of_tuples(create_session):
-    session = create_session(headers=[("h1", "header1"),
-                                      ("h2", "header2"),
-                                      ("h3", "header3")])
+async def test_init_headers_list_of_tuples(create_session) -> None:
+    session = await create_session(headers=[("h1", "header1"),
+                                            ("h2", "header2"),
+                                            ("h3", "header3")])
     assert (session._default_headers ==
             CIMultiDict([("h1", "header1"),
                          ("h2", "header2"),
                          ("h3", "header3")]))
 
 
-def test_init_headers_MultiDict(create_session):
-    session = create_session(headers=MultiDict([("h1", "header1"),
-                                                ("h2", "header2"),
-                                                ("h3", "header3")]))
+async def test_init_headers_MultiDict(create_session) -> None:
+    session = await create_session(headers=MultiDict([("h1", "header1"),
+                                                      ("h2", "header2"),
+                                                      ("h3", "header3")]))
     assert (session._default_headers ==
             CIMultiDict([("H1", "header1"),
                          ("H2", "header2"),
                          ("H3", "header3")]))
 
 
-def test_init_headers_list_of_tuples_with_duplicates(create_session):
-    session = create_session(headers=[("h1", "header11"),
-                                      ("h2", "header21"),
-                                      ("h1", "header12")])
+async def test_init_headers_list_of_tuples_with_duplicates(
+        create_session) -> None:
+    session = await create_session(headers=[("h1", "header11"),
+                                            ("h2", "header21"),
+                                            ("h1", "header12")])
     assert (session._default_headers ==
             CIMultiDict([("H1", "header11"),
                          ("H2", "header21"),
                          ("H1", "header12")]))
 
 
-def test_init_cookies_with_simple_dict(create_session):
-    session = create_session(cookies={"c1": "cookie1",
-                                      "c2": "cookie2"})
+async def test_init_cookies_with_simple_dict(create_session) -> None:
+    session = await create_session(cookies={"c1": "cookie1",
+                                            "c2": "cookie2"})
     cookies = session.cookie_jar.filter_cookies()
     assert set(cookies) == {'c1', 'c2'}
     assert cookies['c1'].value == 'cookie1'
     assert cookies['c2'].value == 'cookie2'
 
 
-def test_init_cookies_with_list_of_tuples(create_session):
-    session = create_session(cookies=[("c1", "cookie1"),
-                                      ("c2", "cookie2")])
+async def test_init_cookies_with_list_of_tuples(create_session) -> None:
+    session = await create_session(cookies=[("c1", "cookie1"),
+                                            ("c2", "cookie2")])
 
     cookies = session.cookie_jar.filter_cookies()
     assert set(cookies) == {'c1', 'c2'}
@@ -120,35 +123,36 @@ def test_init_cookies_with_list_of_tuples(create_session):
     assert cookies['c2'].value == 'cookie2'
 
 
-def test_merge_headers(create_session):
+async def test_merge_headers(create_session) -> None:
         # Check incoming simple dict
-    session = create_session(headers={"h1": "header1",
-                                      "h2": "header2"})
+    session = await create_session(headers={"h1": "header1",
+                                            "h2": "header2"})
     headers = session._prepare_headers({"h1": "h1"})
 
     assert isinstance(headers, CIMultiDict)
     assert headers == {"h1": "h1", "h2": "header2"}
 
 
-def test_merge_headers_with_multi_dict(create_session):
-    session = create_session(headers={"h1": "header1",
-                                      "h2": "header2"})
+async def test_merge_headers_with_multi_dict(create_session) -> None:
+    session = await create_session(headers={"h1": "header1",
+                                            "h2": "header2"})
     headers = session._prepare_headers(MultiDict([("h1", "h1")]))
     assert isinstance(headers, CIMultiDict)
     assert headers == {"h1": "h1", "h2": "header2"}
 
 
-def test_merge_headers_with_list_of_tuples(create_session):
-    session = create_session(headers={"h1": "header1",
-                                      "h2": "header2"})
+async def test_merge_headers_with_list_of_tuples(create_session) -> None:
+    session = await create_session(headers={"h1": "header1",
+                                            "h2": "header2"})
     headers = session._prepare_headers([("h1", "h1")])
     assert isinstance(headers, CIMultiDict)
     assert headers == {"h1": "h1", "h2": "header2"}
 
 
-def test_merge_headers_with_list_of_tuples_duplicated_names(create_session):
-    session = create_session(headers={"h1": "header1",
-                                      "h2": "header2"})
+async def test_merge_headers_with_list_of_tuples_duplicated_names(
+        create_session) -> None:
+    session = await create_session(headers={"h1": "header1",
+                                            "h2": "header2"})
 
     headers = session._prepare_headers([("h1", "v1"),
                                         ("h1", "v2")])
@@ -159,7 +163,7 @@ def test_merge_headers_with_list_of_tuples_duplicated_names(create_session):
                                              ("h2", "header2")]
 
 
-def test_http_GET(session, params):
+def test_http_GET(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.get("http://test.example.com",
                     params={"x": 1},
@@ -172,7 +176,7 @@ def test_http_GET(session, params):
                                            **params)]
 
 
-def test_http_OPTIONS(session, params):
+def test_http_OPTIONS(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.options("http://opt.example.com",
                         params={"x": 2},
@@ -185,7 +189,7 @@ def test_http_OPTIONS(session, params):
                                            **params)]
 
 
-def test_http_HEAD(session, params):
+def test_http_HEAD(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.head("http://head.example.com",
                      params={"x": 2},
@@ -198,7 +202,7 @@ def test_http_HEAD(session, params):
                                            **params)]
 
 
-def test_http_POST(session, params):
+def test_http_POST(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.post("http://post.example.com",
                      params={"x": 2},
@@ -212,7 +216,7 @@ def test_http_POST(session, params):
                                            **params)]
 
 
-def test_http_PUT(session, params):
+def test_http_PUT(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.put("http://put.example.com",
                     params={"x": 2},
@@ -226,7 +230,7 @@ def test_http_PUT(session, params):
                                            **params)]
 
 
-def test_http_PATCH(session, params):
+def test_http_PATCH(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.patch("http://patch.example.com",
                       params={"x": 2},
@@ -240,7 +244,7 @@ def test_http_PATCH(session, params):
                                            **params)]
 
 
-def test_http_DELETE(session, params):
+def test_http_DELETE(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.delete("http://delete.example.com",
                        params={"x": 2},
@@ -253,24 +257,24 @@ def test_http_DELETE(session, params):
                                            **params)]
 
 
-async def test_close(create_session, connector):
-    session = create_session(connector=connector)
+async def test_close(create_session, connector) -> None:
+    session = await create_session(connector=connector)
 
     await session.close()
     assert session.connector is None
     assert connector.closed
 
 
-async def test_closed(session):
+async def test_closed(session) -> None:
     assert not session.closed
     await session.close()
     assert session.closed
 
 
-async def test_connector(create_session, loop, mocker):
+async def test_connector(create_session, loop, mocker) -> None:
     connector = TCPConnector(loop=loop)
     mocker.spy(connector, 'close')
-    session = create_session(connector=connector)
+    session = await create_session(connector=connector)
     assert session.connector is connector
 
     await session.close()
@@ -278,8 +282,8 @@ async def test_connector(create_session, loop, mocker):
     connector.close()
 
 
-async def test_create_connector(create_session, loop, mocker):
-    session = create_session()
+async def test_create_connector(create_session, loop, mocker) -> None:
+    session = await create_session()
     connector = session.connector
     mocker.spy(session.connector, 'close')
 
@@ -287,19 +291,25 @@ async def test_create_connector(create_session, loop, mocker):
     assert connector.close.called
 
 
-def test_connector_loop(loop):
+def test_connector_loop(loop) -> None:
     with contextlib.ExitStack() as stack:
         another_loop = asyncio.new_event_loop()
         stack.enter_context(contextlib.closing(another_loop))
-        connector = TCPConnector(loop=another_loop)
+
+        async def make_connector():
+            return TCPConnector()
+        connector = another_loop.run_until_complete(make_connector())
+
         stack.enter_context(contextlib.closing(connector))
         with pytest.raises(RuntimeError) as ctx:
-            ClientSession(connector=connector, loop=loop)
+            async def make_sess():
+                return ClientSession(connector=connector, loop=loop)
+            loop.run_until_complete(make_sess())
         assert re.match("Session and connector has to use same event loop",
                         str(ctx.value))
 
 
-def test_detach(session):
+def test_detach(session) -> None:
     conn = session.connector
     try:
         assert not conn.closed
@@ -311,21 +321,21 @@ def test_detach(session):
         conn.close()
 
 
-async def test_request_closed_session(session):
+async def test_request_closed_session(session) -> None:
     await session.close()
     with pytest.raises(RuntimeError):
         await session.request('get', '/')
 
 
-def test_close_flag_for_closed_connector(session):
+def test_close_flag_for_closed_connector(session) -> None:
     conn = session.connector
     assert not session.closed
     conn.close()
     assert session.closed
 
 
-async def test_double_close(connector, create_session):
-    session = create_session(connector=connector)
+async def test_double_close(connector, create_session) -> None:
+    session = await create_session(connector=connector)
 
     await session.close()
     assert session.connector is None
@@ -334,17 +344,42 @@ async def test_double_close(connector, create_session):
     assert connector.closed
 
 
-def test_del(connector, loop):
+async def test_del(connector, loop) -> None:
+    loop.set_debug(False)
     # N.B. don't use session fixture, it stores extra reference internally
     session = ClientSession(connector=connector, loop=loop)
-    loop.set_exception_handler(lambda loop, ctx: None)
+    logs = []
+    loop.set_exception_handler(lambda loop, ctx: logs.append(ctx))
 
     with pytest.warns(ResourceWarning):
         del session
         gc.collect()
 
+    assert len(logs) == 1
+    expected = {'client_session': mock.ANY,
+                'message': 'Unclosed client session'}
+    assert logs[0] == expected
 
-def test_context_manager(connector, loop):
+
+async def test_del_debug(connector, loop) -> None:
+    loop.set_debug(True)
+    # N.B. don't use session fixture, it stores extra reference internally
+    session = ClientSession(connector=connector, loop=loop)
+    logs = []
+    loop.set_exception_handler(lambda loop, ctx: logs.append(ctx))
+
+    with pytest.warns(ResourceWarning):
+        del session
+        gc.collect()
+
+    assert len(logs) == 1
+    expected = {'client_session': mock.ANY,
+                'message': 'Unclosed client session',
+                'source_traceback': mock.ANY}
+    assert logs[0] == expected
+
+
+async def test_context_manager(connector, loop) -> None:
     with pytest.raises(TypeError):
         with ClientSession(loop=loop, connector=connector) as session:
             pass
@@ -352,7 +387,7 @@ def test_context_manager(connector, loop):
         assert session.closed
 
 
-async def test_borrow_connector_loop(connector, create_session, loop):
+async def test_borrow_connector_loop(connector, create_session, loop) -> None:
     session = ClientSession(connector=connector, loop=None)
     try:
         assert session._loop, loop
@@ -360,14 +395,14 @@ async def test_borrow_connector_loop(connector, create_session, loop):
         await session.close()
 
 
-async def test_reraise_os_error(create_session):
+async def test_reraise_os_error(create_session) -> None:
     err = OSError(1, "permission error")
     req = mock.Mock()
     req_factory = mock.Mock(return_value=req)
     req.send = mock.Mock(side_effect=err)
-    session = create_session(request_class=req_factory)
+    session = await create_session(request_class=req_factory)
 
-    async def create_connection(req, traces=None):
+    async def create_connection(req, traces, timeout):
         # return self.transport, self.protocol
         return mock.Mock()
     session._connector._create_connection = create_connection
@@ -380,7 +415,7 @@ async def test_reraise_os_error(create_session):
     assert e.strerror == err.strerror
 
 
-async def test_close_conn_on_error(create_session):
+async def test_close_conn_on_error(create_session) -> None:
     class UnexpectedException(BaseException):
         pass
 
@@ -388,17 +423,17 @@ async def test_close_conn_on_error(create_session):
     req = mock.Mock()
     req_factory = mock.Mock(return_value=req)
     req.send = mock.Mock(side_effect=err)
-    session = create_session(request_class=req_factory)
+    session = await create_session(request_class=req_factory)
 
     connections = []
     original_connect = session._connector.connect
 
-    async def connect(req, traces=None):
-        conn = await original_connect(req, traces=traces)
+    async def connect(req, traces, timeout):
+        conn = await original_connect(req, traces, timeout)
         connections.append(conn)
         return conn
 
-    async def create_connection(req, traces=None):
+    async def create_connection(req, traces, timeout):
         # return self.transport, self.protocol
         conn = mock.Mock()
         return conn
@@ -417,7 +452,7 @@ async def test_close_conn_on_error(create_session):
         c.__del__()
 
 
-async def test_cookie_jar_usage(loop, aiohttp_client):
+async def test_cookie_jar_usage(loop, aiohttp_client) -> None:
     req_url = None
 
     jar = mock.Mock()
@@ -458,18 +493,19 @@ async def test_cookie_jar_usage(loop, aiohttp_client):
     assert resp_cookies["response"].value == "resp_value"
 
 
-def test_session_default_version(loop):
+async def test_session_default_version(loop) -> None:
     session = aiohttp.ClientSession(loop=loop)
     assert session.version == aiohttp.HttpVersion11
 
 
-async def test_session_loop(loop):
+async def test_session_loop(loop) -> None:
     session = aiohttp.ClientSession(loop=loop)
-    assert session.loop is loop
+    with pytest.warns(DeprecationWarning):
+        assert session.loop is loop
     await session.close()
 
 
-def test_proxy_str(session, params):
+def test_proxy_str(session, params) -> None:
     with mock.patch("aiohttp.client.ClientSession._request") as patched:
         session.get("http://test.example.com",
                     proxy='http://proxy.com',
@@ -482,20 +518,7 @@ def test_proxy_str(session, params):
                                            **params)]
 
 
-def test_client_session_implicit_loop_warn():
-    loop = asyncio.new_event_loop()
-    asyncio.set_event_loop(loop)
-
-    with pytest.warns(UserWarning):
-        session = aiohttp.ClientSession()
-        assert session._loop is loop
-        loop.run_until_complete(session.close())
-
-    asyncio.set_event_loop(None)
-    loop.close()
-
-
-async def test_request_tracing(loop, aiohttp_client):
+async def test_request_tracing(loop, aiohttp_client) -> None:
     async def handler(request):
         return web.json_response({'ok': True})
 
@@ -559,7 +582,7 @@ async def test_request_tracing(loop, aiohttp_client):
             {'ok': True}).encode('utf8')
 
 
-async def test_request_tracing_exception(loop):
+async def test_request_tracing_exception(loop) -> None:
     on_request_end = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
     on_request_exception = mock.Mock(
         side_effect=asyncio.coroutine(mock.Mock())
@@ -598,7 +621,7 @@ async def test_request_tracing_exception(loop):
         assert not on_request_end.called
 
 
-async def test_request_tracing_interpose_headers(loop, aiohttp_client):
+async def test_request_tracing_interpose_headers(loop, aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -634,7 +657,7 @@ async def test_request_tracing_interpose_headers(loop, aiohttp_client):
 
 @pytest.mark.skipif(not PY_36,
                     reason="Python 3.6+ required")
-def test_client_session_inheritance():
+def test_client_session_inheritance() -> None:
     with pytest.warns(DeprecationWarning):
         class A(ClientSession):
             pass
@@ -642,7 +665,47 @@ def test_client_session_inheritance():
 
 @pytest.mark.skipif(not DEBUG,
                     reason="The check is applied in DEBUG mode only")
-def test_client_session_custom_attr(loop):
+async def test_client_session_custom_attr(loop) -> None:
     session = ClientSession(loop=loop)
     with pytest.warns(DeprecationWarning):
         session.custom = None
+
+
+async def test_client_session_timeout_args(loop) -> None:
+    session1 = ClientSession(loop=loop)
+    assert session1._timeout == client.DEFAULT_TIMEOUT
+
+    with pytest.warns(DeprecationWarning):
+        session2 = ClientSession(loop=loop,
+                                 read_timeout=20*60,
+                                 conn_timeout=30*60)
+    assert session2._timeout == client.ClientTimeout(total=20*60,
+                                                     connect=30*60)
+
+    with pytest.raises(ValueError):
+        ClientSession(loop=loop,
+                      timeout=client.ClientTimeout(total=10*60),
+                      read_timeout=20*60)
+
+    with pytest.raises(ValueError):
+        ClientSession(loop=loop,
+                      timeout=client.ClientTimeout(total=10 * 60),
+                      conn_timeout=30 * 60)
+
+
+async def test_requote_redirect_url_default() -> None:
+    session = ClientSession()
+    assert session.requote_redirect_url
+
+
+async def test_requote_redirect_url_default_disable() -> None:
+    session = ClientSession(requote_redirect_url=False)
+    assert not session.requote_redirect_url
+
+
+async def test_requote_redirect_setter() -> None:
+    session = ClientSession()
+    assert session.requote_redirect_url
+    with pytest.warns(DeprecationWarning):
+        session.requote_redirect_url = False
+    assert not session.requote_redirect_url
