@@ -1,7 +1,6 @@
 """Tests for aiohttp/protocol.py"""
 
 import asyncio
-import unittest
 import zlib
 from unittest import mock
 
@@ -25,9 +24,9 @@ REQUEST_PARSERS = [HttpRequestParserPy]
 RESPONSE_PARSERS = [HttpResponseParserPy]
 
 try:
-    from aiohttp import _http_parser
-    REQUEST_PARSERS.append(_http_parser.HttpRequestParserC)
-    RESPONSE_PARSERS.append(_http_parser.HttpResponseParserC)
+    from aiohttp.http_parser import HttpRequestParserC, HttpResponseParserC
+    REQUEST_PARSERS.append(HttpRequestParserC)
+    RESPONSE_PARSERS.append(HttpResponseParserC)
 except ImportError:  # pragma: no cover
     pass
 
@@ -40,7 +39,10 @@ def protocol():
 @pytest.fixture(params=REQUEST_PARSERS)
 def parser(loop, protocol, request):
     """Parser implementations"""
-    return request.param(protocol, loop, 8190, 32768, 8190)
+    return request.param(protocol, loop,
+                         max_line_size=8190,
+                         max_headers=32768,
+                         max_field_size=8190)
 
 
 @pytest.fixture(params=REQUEST_PARSERS)
@@ -52,7 +54,10 @@ def request_cls(request):
 @pytest.fixture(params=RESPONSE_PARSERS)
 def response(loop, protocol, request):
     """Parser implementations"""
-    return request.param(protocol, loop, 8190, 32768, 8190)
+    return request.param(protocol, loop,
+                         max_line_size=8190,
+                         max_headers=32768,
+                         max_field_size=8190)
 
 
 @pytest.fixture(params=RESPONSE_PARSERS)
@@ -61,7 +66,12 @@ def response_cls(request):
     return request.param
 
 
-def test_parse_headers(parser):
+@pytest.fixture
+def stream():
+    return mock.Mock()
+
+
+def test_parse_headers(parser) -> None:
     text = b'''GET /test HTTP/1.1\r
 test: line\r
  continue\r
@@ -81,7 +91,7 @@ test2: data\r
     assert not msg.upgrade
 
 
-def test_parse(parser):
+def test_parse(parser) -> None:
     text = b'GET /test HTTP/1.1\r\n\r\n'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
@@ -93,7 +103,7 @@ def test_parse(parser):
     assert msg.version == (1, 1)
 
 
-async def test_parse_body(parser):
+async def test_parse_body(parser) -> None:
     text = b'GET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
@@ -102,7 +112,7 @@ async def test_parse_body(parser):
     assert body == b'body'
 
 
-async def test_parse_body_with_CRLF(parser):
+async def test_parse_body_with_CRLF(parser) -> None:
     text = b'\r\nGET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
@@ -111,7 +121,7 @@ async def test_parse_body_with_CRLF(parser):
     assert body == b'body'
 
 
-def test_parse_delayed(parser):
+def test_parse_delayed(parser) -> None:
     text = b'GET /test HTTP/1.1\r\n'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 0
@@ -123,7 +133,7 @@ def test_parse_delayed(parser):
     assert msg.method == 'GET'
 
 
-def test_headers_multi_feed(parser):
+def test_headers_multi_feed(parser) -> None:
     text1 = b'GET /test HTTP/1.1\r\n'
     text2 = b'test: line\r'
     text3 = b'\n continue\r\n\r\n'
@@ -145,7 +155,7 @@ def test_headers_multi_feed(parser):
     assert not msg.upgrade
 
 
-def test_headers_split_field(parser):
+def test_headers_split_field(parser) -> None:
     text1 = b'GET /test HTTP/1.1\r\n'
     text2 = b't'
     text3 = b'es'
@@ -166,7 +176,7 @@ def test_headers_split_field(parser):
     assert not msg.upgrade
 
 
-def test_parse_headers_multi(parser):
+def test_parse_headers_multi(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'Set-Cookie: c1=cookie1\r\n'
             b'Set-Cookie: c2=cookie2\r\n\r\n')
@@ -183,21 +193,21 @@ def test_parse_headers_multi(parser):
     assert msg.compression is None
 
 
-def test_conn_default_1_0(parser):
+def test_conn_default_1_0(parser) -> None:
     text = b'GET /test HTTP/1.0\r\n\r\n'
     messages, upgrade, tail = parser.feed_data(text)
     msg = messages[0][0]
     assert msg.should_close
 
 
-def test_conn_default_1_1(parser):
+def test_conn_default_1_1(parser) -> None:
     text = b'GET /test HTTP/1.1\r\n\r\n'
     messages, upgrade, tail = parser.feed_data(text)
     msg = messages[0][0]
     assert not msg.should_close
 
 
-def test_conn_close(parser):
+def test_conn_close(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'connection: close\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -205,7 +215,7 @@ def test_conn_close(parser):
     assert msg.should_close
 
 
-def test_conn_close_1_0(parser):
+def test_conn_close_1_0(parser) -> None:
     text = (b'GET /test HTTP/1.0\r\n'
             b'connection: close\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -213,7 +223,7 @@ def test_conn_close_1_0(parser):
     assert msg.should_close
 
 
-def test_conn_keep_alive_1_0(parser):
+def test_conn_keep_alive_1_0(parser) -> None:
     text = (b'GET /test HTTP/1.0\r\n'
             b'connection: keep-alive\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -221,7 +231,7 @@ def test_conn_keep_alive_1_0(parser):
     assert not msg.should_close
 
 
-def test_conn_keep_alive_1_1(parser):
+def test_conn_keep_alive_1_1(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'connection: keep-alive\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -229,7 +239,7 @@ def test_conn_keep_alive_1_1(parser):
     assert not msg.should_close
 
 
-def test_conn_other_1_0(parser):
+def test_conn_other_1_0(parser) -> None:
     text = (b'GET /test HTTP/1.0\r\n'
             b'connection: test\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -237,7 +247,7 @@ def test_conn_other_1_0(parser):
     assert msg.should_close
 
 
-def test_conn_other_1_1(parser):
+def test_conn_other_1_1(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'connection: test\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -245,7 +255,7 @@ def test_conn_other_1_1(parser):
     assert not msg.should_close
 
 
-def test_request_chunked(parser):
+def test_request_chunked(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'transfer-encoding: chunked\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -255,7 +265,7 @@ def test_request_chunked(parser):
     assert isinstance(payload, streams.StreamReader)
 
 
-def test_conn_upgrade(parser):
+def test_conn_upgrade(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'connection: upgrade\r\n'
             b'upgrade: websocket\r\n\r\n')
@@ -266,7 +276,7 @@ def test_conn_upgrade(parser):
     assert upgrade
 
 
-def test_compression_empty(parser):
+def test_compression_empty(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: \r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -274,7 +284,7 @@ def test_compression_empty(parser):
     assert msg.compression is None
 
 
-def test_compression_deflate(parser):
+def test_compression_deflate(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: deflate\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -282,7 +292,7 @@ def test_compression_deflate(parser):
     assert msg.compression == 'deflate'
 
 
-def test_compression_gzip(parser):
+def test_compression_gzip(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: gzip\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -291,7 +301,7 @@ def test_compression_gzip(parser):
 
 
 @pytest.mark.skipif(brotli is None, reason="brotli is not installed")
-def test_compression_brotli(parser):
+def test_compression_brotli(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: br\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -299,7 +309,7 @@ def test_compression_brotli(parser):
     assert msg.compression == 'br'
 
 
-def test_compression_unknown(parser):
+def test_compression_unknown(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-encoding: compress\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -307,7 +317,7 @@ def test_compression_unknown(parser):
     assert msg.compression is None
 
 
-def test_headers_connect(parser):
+def test_headers_connect(parser) -> None:
     text = (b'CONNECT www.google.com HTTP/1.1\r\n'
             b'content-length: 0\r\n\r\n')
     messages, upgrade, tail = parser.feed_data(text)
@@ -316,7 +326,7 @@ def test_headers_connect(parser):
     assert isinstance(payload, streams.StreamReader)
 
 
-def test_headers_old_websocket_key1(parser):
+def test_headers_old_websocket_key1(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'SEC-WEBSOCKET-KEY1: line\r\n\r\n')
 
@@ -324,7 +334,7 @@ def test_headers_old_websocket_key1(parser):
         parser.feed_data(text)
 
 
-def test_headers_content_length_err_1(parser):
+def test_headers_content_length_err_1(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-length: line\r\n\r\n')
 
@@ -332,7 +342,7 @@ def test_headers_content_length_err_1(parser):
         parser.feed_data(text)
 
 
-def test_headers_content_length_err_2(parser):
+def test_headers_content_length_err_2(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'content-length: -1\r\n\r\n')
 
@@ -340,14 +350,14 @@ def test_headers_content_length_err_2(parser):
         parser.feed_data(text)
 
 
-def test_invalid_header(parser):
+def test_invalid_header(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'test line\r\n\r\n')
     with pytest.raises(http_exceptions.BadHttpMessage):
         parser.feed_data(text)
 
 
-def test_invalid_name(parser):
+def test_invalid_name(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'test[]: line\r\n\r\n')
 
@@ -355,48 +365,121 @@ def test_invalid_name(parser):
         parser.feed_data(text)
 
 
-def test_max_header_field_size(parser):
-    name = b'test' * 10 * 1024
+@pytest.mark.parametrize('size', [40960, 8191])
+def test_max_header_field_size(parser, size) -> None:
+    name = b't' * size
     text = (b'GET /test HTTP/1.1\r\n' + name + b':data\r\n\r\n')
 
-    with pytest.raises(http_exceptions.LineTooLong):
+    match = ("400, message='Got more than 8190 bytes \\({}\\) when reading"
+             .format(size))
+    with pytest.raises(http_exceptions.LineTooLong, match=match):
         parser.feed_data(text)
 
 
-def test_max_header_value_size(parser):
-    name = b'test' * 10 * 1024
+def test_max_header_field_size_under_limit(parser) -> None:
+    name = b't' * 8190
+    text = (b'GET /test HTTP/1.1\r\n' + name + b':data\r\n\r\n')
+
+    messages, upgrade, tail = parser.feed_data(text)
+    msg = messages[0][0]
+    assert msg.method == 'GET'
+    assert msg.path == '/test'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict({name.decode(): 'data'})
+    assert msg.raw_headers == ((name, b'data'),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/test')
+
+
+@pytest.mark.parametrize('size', [40960, 8191])
+def test_max_header_value_size(parser, size) -> None:
+    name = b't' * size
     text = (b'GET /test HTTP/1.1\r\n'
             b'data:' + name + b'\r\n\r\n')
 
-    with pytest.raises(http_exceptions.LineTooLong):
+    match = ("400, message='Got more than 8190 bytes \\({}\\) when reading"
+             .format(size))
+    with pytest.raises(http_exceptions.LineTooLong, match=match):
         parser.feed_data(text)
 
 
-def test_max_header_value_size_continuation(parser):
-    name = b'test' * 10 * 1024
+def test_max_header_value_size_under_limit(parser) -> None:
+    value = b'A' * 8190
+    text = (b'GET /test HTTP/1.1\r\n'
+            b'data:' + value + b'\r\n\r\n')
+
+    messages, upgrade, tail = parser.feed_data(text)
+    msg = messages[0][0]
+    assert msg.method == 'GET'
+    assert msg.path == '/test'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict({'data': value.decode()})
+    assert msg.raw_headers == ((b'data', value),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/test')
+
+
+@pytest.mark.parametrize('size', [40965, 8191])
+def test_max_header_value_size_continuation(parser, size) -> None:
+    name = b'T' * (size - 5)
     text = (b'GET /test HTTP/1.1\r\n'
             b'data: test\r\n ' + name + b'\r\n\r\n')
 
-    with pytest.raises(http_exceptions.LineTooLong):
+    match = ("400, message='Got more than 8190 bytes \\({}\\) when reading"
+             .format(size))
+    with pytest.raises(http_exceptions.LineTooLong, match=match):
         parser.feed_data(text)
 
 
-def test_http_request_parser(parser):
+def test_max_header_value_size_continuation_under_limit(parser) -> None:
+    value = b'A' * 8185
+    text = (b'GET /test HTTP/1.1\r\n'
+            b'data: test\r\n ' + value + b'\r\n\r\n')
+
+    messages, upgrade, tail = parser.feed_data(text)
+    msg = messages[0][0]
+    assert msg.method == 'GET'
+    assert msg.path == '/test'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict({'data': 'test ' + value.decode()})
+    assert msg.raw_headers == ((b'data', b'test ' + value),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/test')
+
+
+def test_http_request_parser(parser) -> None:
     text = b'GET /path HTTP/1.1\r\n\r\n'
     messages, upgrade, tail = parser.feed_data(text)
     msg = messages[0][0]
 
-    assert msg == ('GET', '/path', (1, 1), CIMultiDict(), (),
-                   False, None, False, False, URL('/path'))
+    assert msg.method == 'GET'
+    assert msg.path == '/path'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict()
+    assert msg.raw_headers == ()
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/path')
 
 
-def test_http_request_bad_status_line(parser):
+def test_http_request_bad_status_line(parser) -> None:
     text = b'getpath \r\n\r\n'
     with pytest.raises(http_exceptions.BadStatusLine):
         parser.feed_data(text)
 
 
-def test_http_request_upgrade(parser):
+def test_http_request_upgrade(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'connection: upgrade\r\n'
             b'upgrade: websocket\r\n\r\n'
@@ -409,53 +492,92 @@ def test_http_request_upgrade(parser):
     assert tail == b'some raw data'
 
 
-def test_http_request_parser_utf8(parser):
+def test_http_request_parser_utf8(parser) -> None:
     text = 'GET /path HTTP/1.1\r\nx-test:тест\r\n\r\n'.encode('utf-8')
     messages, upgrade, tail = parser.feed_data(text)
     msg = messages[0][0]
 
-    assert msg == ('GET', '/path', (1, 1),
-                   CIMultiDict([('X-TEST', 'тест')]),
-                   ((b'x-test', 'тест'.encode('utf-8')),),
-                   False, None, False, False, URL('/path'))
+    assert msg.method == 'GET'
+    assert msg.path == '/path'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict([('X-TEST', 'тест')])
+    assert msg.raw_headers == ((b'x-test', 'тест'.encode('utf-8')),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/path')
 
 
-def test_http_request_parser_non_utf8(parser):
+def test_http_request_parser_non_utf8(parser) -> None:
     text = 'GET /path HTTP/1.1\r\nx-test:тест\r\n\r\n'.encode('cp1251')
     msg = parser.feed_data(text)[0][0][0]
 
-    assert msg == ('GET', '/path', (1, 1),
-                   CIMultiDict([('X-TEST', 'тест'.encode('cp1251').decode(
-                       'utf-8', 'surrogateescape'))]),
-                   ((b'x-test', 'тест'.encode('cp1251')),),
-                   False, None, False, False, URL('/path'))
+    assert msg.method == 'GET'
+    assert msg.path == '/path'
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict([('X-TEST', 'тест'.encode('cp1251')
+                                        .decode('utf8', 'surrogateescape'))])
+    assert msg.raw_headers == ((b'x-test', 'тест'.encode('cp1251')),)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/path')
 
 
-def test_http_request_parser_two_slashes(parser):
+def test_http_request_parser_two_slashes(parser) -> None:
     text = b'GET //path HTTP/1.1\r\n\r\n'
     msg = parser.feed_data(text)[0][0][0]
 
-    assert msg[:-1] == ('GET', '//path', (1, 1), CIMultiDict(), (),
-                        False, None, False, False)
+    assert msg.method == 'GET'
+    assert msg.path == '//path'
+    assert msg.version == (1, 1)
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
 
 
-def test_http_request_parser_bad_method(parser):
+def test_http_request_parser_bad_method(parser) -> None:
     with pytest.raises(http_exceptions.BadStatusLine):
-        parser.feed_data(b'!12%()+=~$ /get HTTP/1.1\r\n\r\n')
+        parser.feed_data(b'=":<G>(e),[T];?" /get HTTP/1.1\r\n\r\n')
 
 
-def test_http_request_parser_bad_version(parser):
+def test_http_request_parser_bad_version(parser) -> None:
     with pytest.raises(http_exceptions.BadHttpMessage):
         parser.feed_data(b'GET //get HT/11\r\n\r\n')
 
 
-def test_http_request_max_status_line(parser):
-    with pytest.raises(http_exceptions.LineTooLong):
+@pytest.mark.parametrize('size', [40965, 8191])
+def test_http_request_max_status_line(parser, size) -> None:
+    path = b't' * (size - 5)
+    match = ("400, message='Got more than 8190 bytes \\({}\\) when reading"
+             .format(size))
+    with pytest.raises(http_exceptions.LineTooLong, match=match):
         parser.feed_data(
-            b'GET /path' + b'test' * 10 * 1024 + b' HTTP/1.1\r\n\r\n')
+            b'GET /path' + path + b' HTTP/1.1\r\n\r\n')
 
 
-def test_http_response_parser_utf8(response):
+def test_http_request_max_status_line_under_limit(parser) -> None:
+    path = b't' * (8190 - 5)
+    messages, upgraded, tail = parser.feed_data(
+        b'GET /path' + path + b' HTTP/1.1\r\n\r\n')
+    msg = messages[0][0]
+
+    assert msg.method == 'GET'
+    assert msg.path == '/path' + path.decode()
+    assert msg.version == (1, 1)
+    assert msg.headers == CIMultiDict()
+    assert msg.raw_headers == ()
+    assert not msg.should_close
+    assert msg.compression is None
+    assert not msg.upgrade
+    assert not msg.chunked
+    assert msg.url == URL('/path' + path.decode())
+
+
+def test_http_response_parser_utf8(response) -> None:
     text = 'HTTP/1.1 200 Ok\r\nx-test:тест\r\n\r\n'.encode('utf-8')
 
     messages, upgraded, tail = response.feed_data(text)
@@ -471,18 +593,32 @@ def test_http_response_parser_utf8(response):
     assert not tail
 
 
-def test_http_response_parser_bad_status_line_too_long(response):
-    with pytest.raises(http_exceptions.LineTooLong):
+@pytest.mark.parametrize('size', [40962, 8191])
+def test_http_response_parser_bad_status_line_too_long(response, size) -> None:
+    reason = b't' * (size - 2)
+    match = ("400, message='Got more than 8190 bytes \\({}\\) when reading"
+             .format(size))
+    with pytest.raises(http_exceptions.LineTooLong, match=match):
         response.feed_data(
-            b'HTTP/1.1 200 Ok' + b'test' * 10 * 1024 + b'\r\n\r\n')
+            b'HTTP/1.1 200 Ok' + reason + b'\r\n\r\n')
 
 
-def test_http_response_parser_bad_version(response):
+def test_http_response_parser_status_line_under_limit(response) -> None:
+    reason = b'O' * 8190
+    messages, upgraded, tail = response.feed_data(
+        b'HTTP/1.1 200 ' + reason + b'\r\n\r\n')
+    msg = messages[0][0]
+    assert msg.version == (1, 1)
+    assert msg.code == 200
+    assert msg.reason == reason.decode()
+
+
+def test_http_response_parser_bad_version(response) -> None:
     with pytest.raises(http_exceptions.BadHttpMessage):
         response.feed_data(b'HT/11 200 Ok\r\n\r\n')
 
 
-def test_http_response_parser_no_reason(response):
+def test_http_response_parser_no_reason(response) -> None:
     msg = response.feed_data(b'HTTP/1.1 200\r\n\r\n')[0][0][0]
 
     assert msg.version == (1, 1)
@@ -490,27 +626,27 @@ def test_http_response_parser_no_reason(response):
     assert not msg.reason
 
 
-def test_http_response_parser_bad(response):
+def test_http_response_parser_bad(response) -> None:
     with pytest.raises(http_exceptions.BadHttpMessage):
         response.feed_data(b'HTT/1\r\n\r\n')
 
 
-def test_http_response_parser_code_under_100(response):
+def test_http_response_parser_code_under_100(response) -> None:
     msg = response.feed_data(b'HTTP/1.1 99 test\r\n\r\n')[0][0][0]
     assert msg.code == 99
 
 
-def test_http_response_parser_code_above_999(response):
+def test_http_response_parser_code_above_999(response) -> None:
     with pytest.raises(http_exceptions.BadHttpMessage):
         response.feed_data(b'HTTP/1.1 9999 test\r\n\r\n')
 
 
-def test_http_response_parser_code_not_int(response):
+def test_http_response_parser_code_not_int(response) -> None:
     with pytest.raises(http_exceptions.BadHttpMessage):
         response.feed_data(b'HTTP/1.1 ttt test\r\n\r\n')
 
 
-def test_http_request_chunked_payload(parser):
+def test_http_request_chunked_payload(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'transfer-encoding: chunked\r\n\r\n')
     msg, payload = parser.feed_data(text)[0][0]
@@ -526,7 +662,7 @@ def test_http_request_chunked_payload(parser):
     assert payload.is_eof()
 
 
-def test_http_request_chunked_payload_and_next_message(parser):
+def test_http_request_chunked_payload_and_next_message(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'transfer-encoding: chunked\r\n\r\n')
     msg, payload = parser.feed_data(text)[0][0]
@@ -548,7 +684,7 @@ def test_http_request_chunked_payload_and_next_message(parser):
     assert not payload2.is_eof()
 
 
-def test_http_request_chunked_payload_chunks(parser):
+def test_http_request_chunked_payload_chunks(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'transfer-encoding: chunked\r\n\r\n')
     msg, payload = parser.feed_data(text)[0][0]
@@ -571,7 +707,7 @@ def test_http_request_chunked_payload_chunks(parser):
     assert payload.is_eof()
 
 
-def test_parse_chunked_payload_chunk_extension(parser):
+def test_parse_chunked_payload_chunk_extension(parser) -> None:
     text = (b'GET /test HTTP/1.1\r\n'
             b'transfer-encoding: chunked\r\n\r\n')
     msg, payload = parser.feed_data(text)[0][0]
@@ -592,7 +728,8 @@ def _test_parse_no_length_or_te_on_post(loop, protocol, request_cls):
     assert payload.is_eof()
 
 
-def test_parse_payload_response_without_body(loop, protocol, response_cls):
+def test_parse_payload_response_without_body(loop, protocol,
+                                             response_cls) -> None:
     parser = response_cls(protocol, loop, response_with_body=False)
     text = (b'HTTP/1.1 200 Ok\r\n'
             b'content-length: 10\r\n\r\n')
@@ -601,7 +738,7 @@ def test_parse_payload_response_without_body(loop, protocol, response_cls):
     assert payload.is_eof()
 
 
-def test_parse_length_payload(response):
+def test_parse_length_payload(response) -> None:
     text = (b'HTTP/1.1 200 Ok\r\n'
             b'content-length: 4\r\n\r\n')
     msg, payload = response.feed_data(text)[0][0]
@@ -615,13 +752,13 @@ def test_parse_length_payload(response):
     assert b'data' == b''.join(d for d in payload._buffer)
 
 
-def test_parse_no_length_payload(parser):
+def test_parse_no_length_payload(parser) -> None:
     text = b'PUT / HTTP/1.1\r\n\r\n'
     msg, payload = parser.feed_data(text)[0][0]
     assert payload.is_eof()
 
 
-def test_partial_url(parser):
+def test_partial_url(parser) -> None:
     messages, upgrade, tail = parser.feed_data(b'GET /te')
     assert len(messages) == 0
     messages, upgrade, tail = parser.feed_data(b'st HTTP/1.1\r\n\r\n')
@@ -635,7 +772,7 @@ def test_partial_url(parser):
     assert payload.is_eof()
 
 
-def test_url_parse_non_strict_mode(parser):
+def test_url_parse_non_strict_mode(parser) -> None:
     payload = 'GET /test/тест HTTP/1.1\r\n\r\n'.encode('utf-8')
     messages, upgrade, tail = parser.feed_data(payload)
     assert len(messages) == 1
@@ -648,30 +785,29 @@ def test_url_parse_non_strict_mode(parser):
     assert payload.is_eof()
 
 
-class TestParsePayload(unittest.TestCase):
+class TestParsePayload:
 
-    def setUp(self):
-        self.stream = mock.Mock()
-        asyncio.set_event_loop(None)
-
-    def test_parse_eof_payload(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_parse_eof_payload(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(out, readall=True)
         p.feed_data(b'data')
         p.feed_eof()
 
-        self.assertTrue(out.is_eof())
-        self.assertEqual([(bytearray(b'data'), 4)], list(out._buffer))
+        assert out.is_eof()
+        assert [(bytearray(b'data'), 4)] == list(out._buffer)
 
-    def test_parse_no_body(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_parse_no_body(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(out, method='PUT')
 
-        self.assertTrue(out.is_eof())
-        self.assertTrue(p.done)
+        assert out.is_eof()
+        assert p.done
 
-    def test_parse_length_payload_eof(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_parse_length_payload_eof(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
 
         p = HttpPayloadParser(out, length=4)
         p.feed_data(b'da')
@@ -679,115 +815,147 @@ class TestParsePayload(unittest.TestCase):
         with pytest.raises(http_exceptions.ContentLengthError):
             p.feed_eof()
 
-    def test_parse_chunked_payload_size_error(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_parse_chunked_payload_size_error(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(out, chunked=True)
-        self.assertRaises(
-            http_exceptions.TransferEncodingError, p.feed_data, b'blah\r\n')
-        self.assertIsInstance(
-            out.exception(), http_exceptions.TransferEncodingError)
+        with pytest.raises(http_exceptions.TransferEncodingError):
+            p.feed_data(b'blah\r\n')
+        assert isinstance(out.exception(),
+                          http_exceptions.TransferEncodingError)
 
-    def test_http_payload_parser_length(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_http_payload_parser_length(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(out, length=2)
         eof, tail = p.feed_data(b'1245')
-        self.assertTrue(eof)
+        assert eof
 
-        self.assertEqual(b'12', b''.join(d for d, _ in out._buffer))
-        self.assertEqual(b'45', tail)
+        assert b'12' == b''.join(d for d, _ in out._buffer)
+        assert b'45' == tail
 
     _comp = zlib.compressobj(wbits=-zlib.MAX_WBITS)
     _COMPRESSED = b''.join([_comp.compress(b'data'), _comp.flush()])
 
-    def test_http_payload_parser_deflate(self):
+    async def test_http_payload_parser_deflate(self, stream) -> None:
         length = len(self._COMPRESSED)
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(
             out, length=length, compression='deflate')
         p.feed_data(self._COMPRESSED)
-        self.assertEqual(b'data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
-    def test_http_payload_parser_deflate_no_wbits(self):
+    async def test_http_payload_parser_deflate_no_wbits(self, stream) -> None:
         comp = zlib.compressobj()
         COMPRESSED = b''.join([comp.compress(b'data'), comp.flush()])
 
         length = len(COMPRESSED)
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(
             out, length=length, compression='deflate')
         p.feed_data(COMPRESSED)
-        self.assertEqual(b'data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
-    def test_http_payload_parser_length_zero(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_http_payload_parser_length_zero(self, stream) -> None:
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(out, length=0)
-        self.assertTrue(p.done)
-        self.assertTrue(out.is_eof())
+        assert p.done
+        assert out.is_eof()
 
     @pytest.mark.skipif(brotli is None, reason="brotli is not installed")
-    def test_http_payload_brotli(self):
+    async def test_http_payload_brotli(self, stream) -> None:
         compressed = brotli.compress(b'brotli data')
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         p = HttpPayloadParser(
             out, length=len(compressed), compression='br')
         p.feed_data(compressed)
-        self.assertEqual(b'brotli data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'brotli data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
 
-class TestDeflateBuffer(unittest.TestCase):
+class TestDeflateBuffer:
 
-    def setUp(self):
-        self.stream = mock.Mock()
-        asyncio.set_event_loop(None)
-
-    def test_feed_data(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_feed_data(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.decompress.return_value = b'line'
 
         dbuf.feed_data(b'data', 4)
-        self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
+        assert [b'line'] == list(d for d, _ in buf._buffer)
 
-    def test_feed_data_err(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_feed_data_err(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         dbuf = DeflateBuffer(buf, 'deflate')
 
         exc = ValueError()
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.decompress.side_effect = exc
 
-        self.assertRaises(
-            http_exceptions.ContentEncodingError, dbuf.feed_data, b'data', 4)
+        with pytest.raises(http_exceptions.ContentEncodingError):
+            dbuf.feed_data(b'data', 4)
 
-    def test_feed_eof(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_feed_eof(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.flush.return_value = b'line'
 
         dbuf.feed_eof()
-        self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
-        self.assertTrue(buf._eof)
+        assert [b'line'] == list(d for d, _ in buf._buffer)
+        assert buf._eof
 
-    def test_feed_eof_err(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_feed_eof_err_deflate(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.flush.return_value = b'line'
         dbuf.decompressor.eof = False
 
-        self.assertRaises(http_exceptions.ContentEncodingError, dbuf.feed_eof)
+        with pytest.raises(http_exceptions.ContentEncodingError):
+            dbuf.feed_eof()
 
-    def test_empty_body(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    async def test_feed_eof_no_err_gzip(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
+        dbuf = DeflateBuffer(buf, 'gzip')
+
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.flush.return_value = b'line'
+        dbuf.decompressor.eof = False
+
+        dbuf.feed_eof()
+        assert [b'line'] == list(d for d, _ in buf._buffer)
+
+    async def test_feed_eof_no_err_brotli(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
+        dbuf = DeflateBuffer(buf, 'br')
+
+        dbuf.decompressor = mock.Mock()
+        dbuf.decompressor.flush.return_value = b'line'
+        dbuf.decompressor.eof = False
+
+        dbuf.feed_eof()
+        assert [b'line'] == list(d for d, _ in buf._buffer)
+
+    async def test_empty_body(self, stream) -> None:
+        buf = aiohttp.FlowControlDataQueue(stream,
+                                           loop=asyncio.get_event_loop())
         dbuf = DeflateBuffer(buf, 'deflate')
         dbuf.feed_eof()
 
-        self.assertTrue(buf.at_eof())
+        assert buf.at_eof()
