@@ -24,39 +24,39 @@ def writer(protocol, transport):
     return WebSocketWriter(protocol, transport, use_mask=False)
 
 
-async def test_pong(writer):
+async def test_pong(writer) -> None:
     await writer.pong()
     writer.transport.write.assert_called_with(b'\x8a\x00')
 
 
-async def test_ping(writer):
+async def test_ping(writer) -> None:
     await writer.ping()
     writer.transport.write.assert_called_with(b'\x89\x00')
 
 
-async def test_send_text(writer):
+async def test_send_text(writer) -> None:
     await writer.send(b'text')
     writer.transport.write.assert_called_with(b'\x81\x04text')
 
 
-async def test_send_binary(writer):
+async def test_send_binary(writer) -> None:
     await writer.send('binary', True)
     writer.transport.write.assert_called_with(b'\x82\x06binary')
 
 
-async def test_send_binary_long(writer):
+async def test_send_binary_long(writer) -> None:
     await writer.send(b'b' * 127, True)
     assert writer.transport.write.call_args[0][0].startswith(b'\x82~\x00\x7fb')
 
 
-async def test_send_binary_very_long(writer):
+async def test_send_binary_very_long(writer) -> None:
     await writer.send(b'b' * 65537, True)
     assert (writer.transport.write.call_args_list[0][0][0] ==
             b'\x82\x7f\x00\x00\x00\x00\x00\x01\x00\x01')
     assert writer.transport.write.call_args_list[1][0][0] == b'b' * 65537
 
 
-async def test_close(writer):
+async def test_close(writer) -> None:
     await writer.close(1001, 'msg')
     writer.transport.write.assert_called_with(b'\x88\x05\x03\xe9msg')
 
@@ -68,7 +68,7 @@ async def test_close(writer):
     writer.transport.write.assert_called_with(b'\x88\x05\x03\xf4msg')
 
 
-async def test_send_text_masked(protocol, transport):
+async def test_send_text_masked(protocol, transport) -> None:
     writer = WebSocketWriter(protocol,
                              transport,
                              use_mask=True,
@@ -77,7 +77,7 @@ async def test_send_text_masked(protocol, transport):
     writer.transport.write.assert_called_with(b'\x81\x84\rg\xb3fy\x02\xcb\x12')
 
 
-async def test_send_compress_text(protocol, transport):
+async def test_send_compress_text(protocol, transport) -> None:
     writer = WebSocketWriter(protocol, transport, compress=15)
     await writer.send(b'text')
     writer.transport.write.assert_called_with(b'\xc1\x06*I\xad(\x01\x00')
@@ -85,7 +85,7 @@ async def test_send_compress_text(protocol, transport):
     writer.transport.write.assert_called_with(b'\xc1\x05*\x01b\x00\x00')
 
 
-async def test_send_compress_text_notakeover(protocol, transport):
+async def test_send_compress_text_notakeover(protocol, transport) -> None:
     writer = WebSocketWriter(protocol,
                              transport,
                              compress=15,
@@ -96,7 +96,7 @@ async def test_send_compress_text_notakeover(protocol, transport):
     writer.transport.write.assert_called_with(b'\xc1\x06*I\xad(\x01\x00')
 
 
-async def test_send_compress_text_per_message(protocol, transport):
+async def test_send_compress_text_per_message(protocol, transport) -> None:
     writer = WebSocketWriter(protocol, transport)
     await writer.send(b'text', compress=15)
     writer.transport.write.assert_called_with(b'\xc1\x06*I\xad(\x01\x00')
